; -------------------------------------------------------------------------------------------
; PROTOCOLO I2C                                                       09/06/2009 * 20/01/2020
; 
; STATUS: FUNCIONA
;
; Este servico fornece funcoes de acesso a dispositivos I2C para a familia 16F.
;
; Na familia 18F use o servico "i2cmssp_service.inc" que usara o modulo MSSP no modo I2C para
; realizar funcoes similares.
;
; -------------------------------------------------------------------------------------------
; ATENCAO:
;          As funcoes deste arquivo foram testadas exaustivamente e funcionam muito bem.
;          Nao perca tempo tentando debugar essas funcoes se o dispositivo I2C nao responder a uma requisicao.
;          Se o dispositivo I2C nao estiver respondendo, tente mudar os pinos SDA e SCL do dispositivo I2C para 
;          outras portas do microcontrolador.
;          Conexoes com fios muito longos tambem causam problemas de comunicacao entre o dispositivo I2C e
;          o microcontrolador.
;                                                      Isso deu canseira 
;                                                         (17/10/2017)
;
; -------------------------------------------------------------------------------------------

; -------------------------------------------------------------------------------------------
; AS SEGUINTES VARIAVEIS E SIMBOLOS DEVEM SER DECLARADOS NO PROGAMA
; -------------------------------------------------------------------------------------------
;PULSE_DUTY     equ       .25            ; duty cycle do pulso de clock em microsegundos
;                                        ; 25us e' ideal para Fosc ate' 20MHZ
;i2c_N0         RES       1              ; sub-rotinas I2C
;i2c_IBUFF      RES       1              ; sub-rotinas I2C - buffer de leitura
;i2c_OBUFF      RES       1              ; sub-rotinas I2C - buffer de gravacao
;#define     i2c_TRIS     TRISA          ; <--- MODIFIQUE - registrado de controle de I/O da porta I2C    
;#define     i2c_PORT     PORTA          ; <--- MODIFIQUE - porta de I/O do I2C
;#define     i2c_SDA      1              ; <--- MODIFIQUE - bit SDA do I2C
;#define     i2c_SCL      2              ; <--- MODIFIQUE - bit SCL do I2C
; -------------------------------------------------------------------------------------------

 ifndef _I2C_PROTO16F
#define _I2C_PROTO16F   1
 endif

 ifndef __DELAY_SERVICE
   #include "..\..\Services\delay_services.inc"
   messg "I2CPROTO SERVICES - Arquivo delay_services.inc foi incluido"
 endif

; -------------------------------------------------------------------------------------------
; Master envia um ACK apos ler um byte no Slave
;
; Este funcao deve ser usada apos o Master receber um byte provindo do Slave.
; Se o Master desejar encerrar a leitura, deve enviar um NACK usando a funcao 'i2c_NACK'
; -------------------------------------------------------------------------------------------
i2cSendAck
i2c_SendACK   bcf      i2c_PORT, i2c_SDA     ; SDA = low (ACK)
              bsf      STATUS, RP0
              bcf      i2c_TRIS, i2c_SDA     ; SDA como saida
              bcf      STATUS, RP0  
              movlw    .10
              call     delay_us 
              call     i2c_CLOCK_PULSE
              bsf      STATUS, RP0
              bsf      i2c_TRIS, i2c_SDA     ; SDA como entrada
              bcf      STATUS, RP0
              movlw    .10
              call     delay_us   
              return 
; -------------------------------------------------------------------------------------------
; Master recebe um ACK apos enviar um byte ao Slave
; -------------------------------------------------------------------------------------------
i2cGetAck     ; Get ACK 
i2cGetACK     ; Get ACK
i2cWaitAck    ; Wait ACK
i2cRecAck     ; Receive ACK
i2c_GetACK    bsf      STATUS, RP0
              bsf      i2c_TRIS, i2c_SDA     ; SDA como entrada
              bcf      i2c_TRIS, i2c_SCL     ; SCL como saida
              bcf      STATUS, RP0      
              bsf      i2c_PORT, i2c_SCL     ; clock high
              movlw    .10
              call     delay_us
              movf     i2c_PORT, w           ; le a porta 
              bcf      i2c_PORT, i2c_SCL     ; clock low
              movwf    i2c_IBUFF             ; salva ACK/NACK
              bsf      STATUS, RP0
              bcf      i2c_TRIS, i2c_SDA     ; SDA como entrada
              bcf      i2c_TRIS, i2c_SCL     ; SCL como saida
              bcf      STATUS, RP0      
              return 

; -------------------------------------------------------------------------------------------
; Master envia um NACK ao finalizar a leitura
; -------------------------------------------------------------------------------------------
i2cSendNack
i2c_NACK      bsf      i2c_PORT, i2c_SDA
              bsf      STATUS, RP0
              bsf      i2c_TRIS, i2c_SDA
              bcf      STATUS, RP0
              bsf      i2c_PORT, i2c_SCL
              movlw    .10
              call     delay_us
              bsf      STATUS, Z               ; SDA = 0 if device acknowledges
              btfsc    i2c_PORT, i2c_SDA
              bcf      STATUS, Z
              bcf      i2c_PORT, i2c_SCL
              movlw    .10
              call     delay_us
              bsf      i2c_PORT, i2c_SCL
              movlw    .10
              call     delay_us
              return

; -------------------------------------------------------------------------------------------
; Leitura
;
; Entrada: Nao ha
;
; Saida  : W e i2c_IBUFF conterao o dado 
; -------------------------------------------------------------------------------------------
i2cGet
i2c_IN_BYTE   clrf     i2c_IBUFF
              movlw    .8
              movwf    i2c_N0

              bsf      STATUS, RP0
              bsf      i2c_TRIS, i2c_SDA   ; SDA ---> entrada
              bcf      i2c_TRIS, i2c_SCL   ; SCL ---> saida
              bcf      STATUS, RP0

i2c_IN_BIT    bsf      i2c_PORT, i2c_SCL   ; SCL Alto
              btfss    i2c_PORT, i2c_SDA   ; leu zero???
              goto     i2c_IN_ZERO         ; SIM -->
             
i2c_IN_ONE    bsf      STATUS, C           ; Carry=1
              rlf      i2c_IBUFF, f        ; salva um bit no buffer
              goto     i2c_CONT_IN         ; contabiliza 

i2c_IN_ZERO   bcf      STATUS, C           ; Carry = 0
              rlf      i2c_IBUFF, f        ; salva o bit no buffer
              
i2c_CONT_IN   bcf      i2c_PORT, i2c_SCL   ; SCL Baixo
              decfsz   i2c_N0, f           ; leu todos os bits???
              goto     i2c_IN_BIT          ; NAO --> continua...

              bsf      STATUS, RP0
              bcf      i2c_TRIS, i2c_SDA   ; SDA ---> saida
              bcf      STATUS, RP0
              movf     i2c_IBUFF, w         ; W --> dado lido
              return

; -------------------------------------------------------------------------------------------
; Esta funcao foi gerada para compatibilidade com outros microcontroladores
; -------------------------------------------------------------------------------------------
i2cIdle       bsf      i2c_PORT, i2c_SDA
              bsf      i2c_PORT, i2c_SCL
              bsf      STATUS, RP0
              bcf      i2c_TRIS, i2c_SDA
              bcf      i2c_TRIS, i2c_SCL
              bcf      STATUS, RP0
              call     i2cWaitLong
              bsf      STATUS, RP0
              bsf      i2c_TRIS, i2c_SDA
              bsf      i2c_TRIS, i2c_SCL
              bcf      STATUS, RP0
              movlw    .10
              call     delay_us
              return

; -------------------------------------------------------------------------------------------
; Envia um byte pelo bus I2C
;
; Obs: Apos chamar esta funcao, deve-se usar a funcao i2c_GetACK ou i2c_NACK
;      antes de prosseguir.
;
; Entrada: W contem o dado
;
; Saida  : Nao ha
;
; -------------------------------------------------------------------------------------------
i2cPut
i2c_OUT_BYTE   movwf   i2c_OBUFF                 ; salva o dado
               movlw   .8                        ; numero de bits a enviar
               movwf   i2c_N0                    ; carrega o contador
               bsf      STATUS, RP0
               bcf      i2c_TRIS, i2c_SDA        ; SDA ---> saida
               bcf      STATUS, RP0
i2c_OUT_BIT    bcf     i2c_PORT, i2c_SDA         ; assume dado=0
               rlf     i2c_OBUFF, f              ; desloca o dado (MSB) para o carry
               btfsc   STATUS, C                 ; dado==0 ?
               bsf     i2c_PORT, i2c_SDA         ; NAO --> dado=1
               call    i2c_CLOCK_PULSE	         ; pulsa o clock para enviar
               decfsz  i2c_N0, f                 ; fim do byte?
               goto    i2c_OUT_BIT               ; NAO --> pega o proximo
               return                            ; fim
; -------------------------------------------------------------------------------------------
; Envia condicao de Start
; -------------------------------------------------------------------------------------------
i2cStart
i2c_START      bsf       i2c_PORT, i2c_SCL
               bsf       i2c_PORT, i2c_SDA
               bsf       STATUS, RP0
               bcf       i2c_TRIS, i2c_SDA
               bcf       i2c_TRIS, i2c_SCL 
               bcf       STATUS, RP0
               movlw     .12
               call      delay_us
               bcf       i2c_PORT, i2c_SDA
               movlw     .10
               call      delay_us
               bcf       i2c_PORT, i2c_SCL
               movlw     .10
               call      delay_us
               return

; -------------------------------------------------------------------------------------------
; Envia condicao de Stop
; -------------------------------------------------------------------------------------------
i2cStop
i2c_STOP       bsf       STATUS, RP0
               bcf       i2c_TRIS, i2c_SDA
               bcf       i2c_TRIS, i2c_SCL 
               bcf       STATUS, RP0

               bsf       i2c_PORT, i2c_SCL
               movlw     .10
               call      delay_us

               bcf       i2c_PORT, i2c_SDA
               movlw     .10
               call      delay_us

               bsf       i2c_PORT, i2c_SDA
               movlw     .10
               call      delay_us

               return


; -------------------------------------------------------------------------------------------
; Emite um pulso de clock (SCL)                                                    21/01/2020
;
; Esta funcao emite um pulso de clock no pino SCL mantendo o sinal alto por 20 microsegundos
; e garantindo pelo menos 20 microsegundos em nivel baixo.
; Esse periodo e suficientemente baixo e foi obtido empiricamente para atender o LCD mais
; lento encontrado na gaveta (display verde JHD 162A). Diminuindo esse tempo o display nao
; funcionara corretamente.
;
; -------------------------------------------------------------------------------------------
i2c_CLOCK_PULSE
               bsf     STATUS, RP0
               bcf     i2c_TRIS, i2c_SCL 
               bcf     STATUS, RP0
               bsf     i2c_PORT, i2c_SCL
               movlw   PULSE_DUTY
               call    delay_us
               bcf     i2c_PORT, i2c_SCL
               movlw   PULSE_DUTY
               call    delay_us
               return		

; -------------------------------------------------------------------------------------------
; Retardo de 5 ciclos de instrucoes
;
; Esta funcao fornece um retardo de 5 ciclos de instrucoes contando com a sua chamada (CALL).
; -------------------------------------------------------------------------------------------
i2cWait       movlw    .10
              call     delay_us
              return                   ; 2 ciclos

; -------------------------------------------------------------------------------------------
; Retardo de 12 milisegundos
; -------------------------------------------------------------------------------------------
i2cWaitLong   movlw    .12
              call     delay_ms
              return

; ----------------------------------------------------------------------
; Funcao: i2cGetWriteAddress                                  06/12/2019
; 
; Dado o endereco real de 7 bits do disposotivo I2C, esta funcao retorna
; o endereco de WRITE de 8 bits.
;
; Entrada: W contem o endereco real de 7 bits do dispositivo I2C.
;
; Saida  : W contera o endereco de WRITE de 8 bits.
;
; ----------------------------------------------------------------------
i2cGetWriteAddress
           movwf   i2c_IBUFF
           rlf     i2c_IBUFF, f
           movf    i2c_IBUFF, w
           andlw   0xFE
           return

; ----------------------------------------------------------------------
; Funcao: i2cGetReadAddress                                   06/12/2019
; 
; Dado o endereco real de 7 bits do disposotivo I2C, esta funcao retorna
; o endereco de READ de 8 bits.
;
; Entrada: W contem o endereco real de 7 bits do dispositivo I2C.
;
; Saida  : W contera o endereco de READ de 8 bits.
;
; ----------------------------------------------------------------------
i2cGetReadAddress
           movwf   i2c_IBUFF  
           rlf     i2c_IBUFF, f
           movf    i2c_IBUFF, w
           iorlw   0x01
           return